#include "zlaps.h"
#ifndef NO_IMAGES
#include "zlaps_xpms.h"
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <sys/time.h>

/* Stupid globals because I'm lazy! */
QTimer *timer;
static struct timeval time_start, time_stop;
QLabel *map;
QLabel *aunit;
QLabel *lunit;
QLabel *sunit;
QLabel *lavg;
QLineEdit *length;
QLCDNumber *secs;
QLCDNumber *aval;
QLCDNumber *sval;
QComboBox *track;
static int RunMode = 0;    /* 0 = off, 1 = on */
static int UnitMode = 0;   /* 0 = english, 1 = metric */
static int CurrentLap = 0;
static float avg_speed = 0;
static float lap_speed = 0;
static float length_data[NUM_TRACKS] = { 0,
2.684, 3.295, 1.540, 0.533,   2.0, 2.937,   1.5, 1.366,   2.5,   1.0,
 1.25, 2.747,  4.24,   1.5, 2.468, 3.061, 2.606,   2.5, 0.686, 2.520,
2.667,   1.5,   1.5,   1.5,   1.5, 2.639, 0.526,  0.75,   2.0,   1.0,
2.068, 3.585, 1.333,   1.0, 1.058, 1.017, 2.831,   1.0,   1.0,   2.5,
 0.75, 3.444, 3.193,   0.4, 4.329, 3.641, 2.660,   1.5,   1.5, 2.454 };

/*
 * Calculate the average(s)...
 */
static void CalcAvg() {
  char msg[32];
  float time, distance;

  sprintf(msg, "%d Lap Average:", CurrentLap );
  lavg->setText(msg);

  strcpy( msg, length->displayText() );
  distance = atof( msg );

  if( (CurrentLap > 0) && (distance > 0) ) {
    time = (time_stop.tv_sec - time_start.tv_sec) +
           (time_stop.tv_usec - time_start.tv_usec )/1000000.0;
    lap_speed = distance/time*60*60;
    avg_speed = (avg_speed*(CurrentLap-1) + lap_speed)/CurrentLap;
    sprintf(msg, "%.2f", lap_speed );
    sval->display( msg );
    sprintf(msg, "%.2f", avg_speed );
    aval->display( msg );
  }
  else { /* reset */
    secs->display( 0 );
    aval->display( 0 );
    sval->display( 0 );
    avg_speed = 0;
    lap_speed = 0;
    time_start.tv_sec = 0;
    time_stop.tv_sec = 0;
  }
}

/* 
 *  Constructs a zlaps which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
zlaps::zlaps( QWidget* parent,  const char* name, WFlags fl )
    : zlapsBase( parent, name, fl )
{
    connect(quit, SIGNAL(clicked()), this, SLOT(GoodBye()));
    connect(control, SIGNAL(clicked()), this, SLOT(StartStop()));
    connect(units, SIGNAL(clicked()), this, SLOT(UnitsChanged()));
    connect(lap, SIGNAL(clicked()), this, SLOT(Lap()));
    connect(help, SIGNAL(clicked()), this, SLOT(Help()));
    /* kludge alert! Saving widgets as globals! */
    map = image_map;
    track = track_box;
    connect( track, SIGNAL( activated( const QString & ) ),
             this, SLOT ( TrackChanged( const QString & ) ) );
    length = length_value;
    lavg = avg_label;
    secs = secs_value;
    aval = avg_value;
    sval = single_value;
    aunit = avg_unit;
    lunit = length_unit;
    sunit = single_unit;
    timer = new QTimer(this);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
zlaps::~zlaps()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Mark a lap and reset timer...
 */
void zlaps::Lap()
{
  if( RunMode ) {
    CurrentLap++;
    CalcAvg();
  }
  else {
    CurrentLap = 0;
  }

  time_start.tv_sec = 0;
  time_stop.tv_sec = 0;
}

/*
 *  Start Running...
 */
void zlaps::Run()
{
  char msg[32];

  if( time_start.tv_sec == 0 ) {
    gettimeofday(&time_start, NULL);
  }
  gettimeofday(&time_stop, NULL);
  sprintf(msg, "%.1f", (time_stop.tv_sec - time_start.tv_sec) +
                       (time_stop.tv_usec - time_start.tv_usec )/1000000.0 );
  secs->display( tr( msg ) );
}

/*
 *  Turn on/off timer...
 */
void zlaps::StartStop()
{

  QPalette p;

  if( RunMode ) {
    control->setText( tr( "Start" ) );
    p = secs->palette();
    p.setColor( QPalette::Normal, QColorGroup::Background, QColor(0, 170, 0) );
    secs->setPalette(p);
    timer->stop();
    RunMode = 0;
    CurrentLap++;
    CalcAvg();
    secs->display( tr( "0" ) );
  }
  else {
    control->setText( tr( "Stop" ) );
    p = secs->palette();
    p.setColor( QPalette::Normal, QColorGroup::Background, QColor(230, 0, 0) );
    secs->setPalette(p);
    RunMode = 1;
    CurrentLap = 0;
    CalcAvg();
    connect( timer, SIGNAL(timeout()), SLOT(Run()) );
    timer->start( 100 );
  }
}

/*
 *  Switch tracks...
 */
void zlaps::TrackChanged(const QString &)
{
  int item;
  char msg[32];
  QPixmap pm;

  CurrentLap = 0;
  CalcAvg();

  item = track->currentItem();

  if( UnitMode == 0 ) {
    sprintf( msg, "%g", length_data[item] );
  }
  else {
    sprintf( msg, "%.3g", length_data[item]*MILES_TO_KILOMETERS );
  }
  length->setText( msg );
  
#ifndef NO_IMAGES
  switch( item ) {
    case 1:
      if ( !QPixmapCache::find("a1", pm) ) {
            QPixmap p( ( const char** ) a1_xpm );
            QPixmapCache::insert("a1", p);
            pm = p;
      }
      break;
    case 2:
      if ( !QPixmapCache::find("albert", pm) ) {
            QPixmap p( ( const char** ) albert_xpm );
            QPixmapCache::insert("albert", p);
            pm = p;
      }
      break;
    case 6:
      if ( !QPixmapCache::find("cat", pm) ) {
            QPixmap p( ( const char** ) cat_xpm );
            QPixmapCache::insert("cat", p);
            pm = p;
      }
      break;
    case 12:
      if ( !QPixmapCache::find("gill", pm) ) {
            QPixmap p( ( const char** ) gill_xpm );
            QPixmapCache::insert("gill", p);
            pm = p;
      }
      break;
    case 13:
      if ( !QPixmapCache::find("hock", pm) ) {
            QPixmap p( ( const char** ) hock_xpm );
            QPixmapCache::insert("hock", p);
            pm = p;
      }
      break;
    case 15:
      if ( !QPixmapCache::find("hung", pm) ) {
            QPixmap p( ( const char** ) hung_xpm );
            QPixmapCache::insert("hung", p);
            pm = p;
      }
      break;
    case 16:
      if ( !QPixmapCache::find("imola", pm) ) {
            QPixmap p( ( const char** ) imola_xpm );
            QPixmapCache::insert("imola", p);
            pm = p;
      }
      break;
    case 17:
      if ( !QPixmapCache::find("indyf1", pm) ) {
            QPixmap p( ( const char** ) indyf1_xpm );
            QPixmapCache::insert("indyf1", p);
            pm = p;
      }
      break;
    case 18:
      if ( !QPixmapCache::find("indy500", pm) ) {
            QPixmap p( ( const char** ) indy500_xpm );
            QPixmapCache::insert("indy500", p);
            pm = p;
      }
      break;
    case 20:
      if ( !QPixmapCache::find("inf", pm) ) {
            QPixmap p( ( const char** ) inf_xpm );
            QPixmapCache::insert("inf", p);
            pm = p;
      }
      break;
      break;
    case 21:
      if ( !QPixmapCache::find("inter", pm) ) {
            QPixmap p( ( const char** ) inter_xpm );
            QPixmapCache::insert("inter", p);
            pm = p;
      }
      break;
      break;
    case 26:
      if ( !QPixmapCache::find("mag", pm) ) {
            QPixmap p( ( const char** ) mag_xpm );
            QPixmapCache::insert("mag", p);
            pm = p;
      }
      break;
    case 31:
      if ( !QPixmapCache::find("mont", pm) ) {
            QPixmap p( ( const char** ) mont_xpm );
            QPixmapCache::insert("mont", p);
            pm = p;
      }
      break;
    case 32:
      if ( !QPixmapCache::find("monz", pm) ) {
            QPixmap p( ( const char** ) monz_xpm );
            QPixmapCache::insert("monz", p);
            pm = p;
      }
      break;
    case 34:
      if ( !QPixmapCache::find("naz", pm) ) {
            QPixmap p( ( const char** ) naz_xpm );
            QPixmapCache::insert("naz", p);
            pm = p;
      }
      break;
    case 37:
      if ( !QPixmapCache::find("nurb", pm) ) {
            QPixmap p( ( const char** ) nurb_xpm );
            QPixmapCache::insert("nurb", p);
            pm = p;
      }
      break;
    case 38:
    case 39:
      if ( !QPixmapCache::find("pike", pm) ) {
            QPixmap p( ( const char** ) pike_xpm );
            QPixmapCache::insert("pike", p);
            pm = p;
      }
      break;
    case 40:
      if ( !QPixmapCache::find("poc", pm) ) {
            QPixmap p( ( const char** ) poc_xpm );
            QPixmapCache::insert("poc", p);
            pm = p;
      }
      break;
    case 42:
      if ( !QPixmapCache::find("sep", pm) ) {
            QPixmap p( ( const char** ) sep_xpm );
            QPixmapCache::insert("sep", p);
            pm = p;
      }
      break;
    case 43:
      if ( !QPixmapCache::find("sil", pm) ) {
            QPixmap p( ( const char** ) sil_xpm );
            QPixmapCache::insert("sil", p);
            pm = p;
      }
      break;
    case 45:
      if ( !QPixmapCache::find("spa", pm) ) {
            QPixmap p( ( const char** ) spa_xpm );
            QPixmapCache::insert("spa", p);
            pm = p;
      }
      break;
    case 46:
      if ( !QPixmapCache::find("suz", pm) ) {
            QPixmap p( ( const char** ) suz_xpm );
            QPixmapCache::insert("suz", p);
            pm = p;
      }
      break;
    case 50:
      if ( !QPixmapCache::find("wat", pm) ) {
            QPixmap p( ( const char** ) wat_xpm );
            QPixmapCache::insert("wat", p);
            pm = p;
      }
      break;
    case 10:
    case 14:
    case 27:
    case 35:
      if ( !QPixmapCache::find("oval", pm) ) {
            QPixmap p( ( const char** ) oval_xpm );
            QPixmapCache::insert("oval", p);
            pm = p;
      }
      break;
    case 3:
    case 5:
    case 7:
    case 9:
    case 22:
    case 23:
    case 24:
    case 25:
    case 29:
    case 33:
    case 36:
    case 41:
    case 47:
    case 48:
      if ( !QPixmapCache::find("ovald", pm) ) {
            QPixmap p( ( const char** ) ovald_xpm );
            QPixmapCache::insert("ovald", p);
            pm = p;
      }
      break;
    case 4:
    case 19:
    case 28:
    case 30:
    case 44:
      if ( !QPixmapCache::find("ovals", pm) ) {
            QPixmap p( ( const char** ) ovals_xpm );
            QPixmapCache::insert("ovals", p);
            pm = p;
      }
      break;
    case 8:
    case 11:
    case 49:
      if ( !QPixmapCache::find("egg", pm) ) {
            QPixmap p( ( const char** ) egg_xpm );
            QPixmapCache::insert("egg", p);
            pm = p;
      }
      break;
    default:
      if ( !QPixmapCache::find("default", pm) ) {
            QPixmap p( ( const char** ) default_xpm );
            QPixmapCache::insert("default", p);
            pm = p;
      }
      break;
  }
  map->setPixmap( pm );
#endif
}

/*
 *  Switch units...
 */
void zlaps::UnitsChanged()
{
  char msg[32];
  int item;
  float l;

  item = track->currentItem();

  if( UnitMode == 0 ) {
    UnitMode = 1;
    units->setText( tr( "English" ) );
    aunit->setText("(kph)");
    sunit->setText("(kph)");
    lunit->setText("(kilometers)");
    if( item > 0 )
      sprintf( msg, "%.3g", length_data[item]*MILES_TO_KILOMETERS );
    else {
      strcpy( msg, length->displayText() );
      l = atof( msg );
      sprintf( msg, "%.3g", l * MILES_TO_KILOMETERS );
    }
    length->setText( msg );
    avg_speed = avg_speed * MILES_TO_KILOMETERS;
    lap_speed = lap_speed * MILES_TO_KILOMETERS;
    sprintf(msg, "%.2f", lap_speed );
    sval->display( msg );
    sprintf(msg, "%.2f", avg_speed );
    aval->display( msg );
  }
  else {
    UnitMode = 0;
    units->setText( tr( "Metric" ) );
    aunit->setText("(mph)");
    sunit->setText("(mph)");
    lunit->setText("(miles)");
    if( item > 0 )
      sprintf( msg, "%g", length_data[item] );
    else {
      strcpy( msg, length->displayText() );
      l = atof( msg );
      sprintf( msg, "%.3g", l * KILOMETERS_TO_MILES );
    }
    length->setText( msg );
    avg_speed = avg_speed * KILOMETERS_TO_MILES;
    lap_speed = lap_speed * KILOMETERS_TO_MILES;
    sprintf(msg, "%.2f", lap_speed );
    sval->display( msg );
    sprintf(msg, "%.2f", avg_speed );
    aval->display( msg );
  }
}

/*
 *  Help...
 */
                                                                               
void zlaps::Help()
{
  QMessageBox::information( this, "zLapSpeed - Help",
                            "Zaurus Lap Speed Calculator\n\n"
                            "Calculate a driver's single\n"
                            "lap average & qualification\n"
                            "lap(s) average. 50 different\n"
                            "race tracks!  Use it for Indy Car,\n"
                            "NASCAR or Formula One!\n\n"
                            "Start/Stop: Turn timer on/off.\n"
                            "Lap: Mark lap & reset timer.\n"
                            "Metric/English: Change units.\n\n"
                            "License: GPL\n\n"
                            "Written by David Brown\n"
                            "<db@cinlug.org>." );
}

/*
 *  Quit...
 */
void zlaps::GoodBye()
{
    close();
}
